import datetime

from djangotools.common.types import date_to_string, parse_date

from baby.app.models.regle import Cycle


class MonthCalendar:
    JOURS = ["Lundi", "Mardi", "Mecredi", "Jeudi", "Vendredi", "Samedi", "Dimanche"]
    JOURS_COURS = ["Lu", "Ma", "Me", "Je", "Ve", "Sa", "Di"]
    MOIS = ["", "Janvier", "Février", "Mars", "Avril", "Mai", "Juin", "Juillet", "Août", "Septembre",
             "Octobre", "Novembre", "Décembre" ]


    def __init__(self,  user, mont_or_date, year=None):
        if isinstance(mont_or_date, str):
            mont_or_date = parse_date(mont_or_date)
        if isinstance(mont_or_date, (datetime.date, datetime.datetime)):
            mont_or_date, year = mont_or_date.month, mont_or_date.year
        assert isinstance(mont_or_date, int) and isinstance(year, int)
        if year < 100: year += 2000
        assert 0 < mont_or_date <= 12
        self.month = mont_or_date
        self.user = user
        self.year = year
        self.month_date = datetime.date(self.year, self.month, 1)

        start = datetime.date(year, mont_or_date, 1)
        # on commence au lundi même s'il est sur le mois précédent
        self.start = start - datetime.timedelta(days=start.weekday())


        end = start + datetime.timedelta(days=32)
        end = end - datetime.timedelta(days=end.day)
        self.end = end + datetime.timedelta(days=6-end.weekday())

        self.cycles = Cycle.from_date_range(user, self.start, self.end)

    @property
    def prev_month(self):
        start = self.month_date - datetime.timedelta(days=1)
        start = start - datetime.timedelta(days=start.day)
        return start

    @property
    def next_month(self):
        end =  self.month_date + datetime.timedelta(days=31)
        return end

    @property
    def prev_calendar(self):
        return self.__class__(self.user, mont_or_date=self.prev_month)

    @property
    def next_calendar(self):
        return self.__class__(self.user, mont_or_date=self.next_month)


    def get_calendar(self):
        ret = []
        line = []
        for i, value in enumerate(self.cycles.iter_dates(self.start, self.end)):
            jour = i%7
            if jour == 0:
                line = []
                ret.append(line)

            line.append(value)
        return ret

    def get_calendar_dict(self):
        ret = []
        line = []

        iterator = self.cycles.iter_dates(self.start, self.end)
        if not self.cycles:
            x = self.start
            iterator = []
            while x <= self.end:
                iterator.append((x, []))
                x = x + datetime.timedelta(days=1)



        for i, (date, tags) in enumerate(iterator):
            jour = i%7
            if jour == 0:
                line = []
                ret.append(line)
            day = {
                "date" : date,
                "day" : date.day,
                "month" : date.month,
                "year" : date.year,
                "current_month" : date.month == self.month,
                "not_current_month" : date.month != self.month,
                Cycle.OVULATION : Cycle.OVULATION in tags,
                Cycle.FERTILITE : Cycle.FERTILITE in tags,
                Cycle.DEBUT : Cycle.DEBUT in tags,
                "classes" : []
            }

            for tag in ["current_month", "not_current_month", Cycle.DEBUT, Cycle.FERTILITE, Cycle.OVULATION]:
                if day[tag]:
                    day["classes"].append(f"tag-{tag}")
            day["classes"] = " ".join(day["classes"])

            line.append(day)
        return ret


    @property
    def json(self):
        return {
            "month" : self.month,
            "month_word" : self.MOIS[self.month],
            "year" : self.start.year,
            "next" : date_to_string(self.next_month)[3:],
            "prev" : date_to_string(self.prev_month)[3:],
            "calendar" : self.get_calendar_dict()
        }

